#ifndef PLOG_LOG_MESSAGE_H
#define PLOG_LOG_MESSAGE_H

#include <sstream>
#include <ctime>
#include <locale>

#include <windows.h>
#include "plog/log_level.h"
#include "plog/context_info.h"

namespace plog
{

#define CONTEXT_INFO(OBJ) plog::context_info(__FILE__, __FUNCTION__, __LINE__,::GetCurrentThreadId(), OBJ)

    //////////////////////////////////////////////////////////////////////////

    class message
    {
        friend bool operator==(const message& msg1, const message& msg2);

    public:
        message(log_level severity, const context_info& context, const char* message = "")
            : locale_("")
            , severity_(severity)
            , stream_(message)
            , context_(context)
        {
        }

        template<typename T>
        message& operator<< (const T& chunk)
        {
            stream_ << chunk;
            return *this;
        }

        message& operator<< (const char* chunk)
        {
            chunk = chunk ? chunk : "(null)";

            stream_ << chunk;
            return *this;
        }

        message& operator<< (char* chunk)
        {
            return operator<< <const char*> (static_cast<const char*>(chunk));
        }

        message& operator<< (const wchar_t* chunk)
        {
            chunk = chunk ? chunk : L"(null)";

            for(; *chunk; ++chunk)
                stream_ << std::use_facet< std::ctype<wchar_t> >(locale_).narrow(*chunk, '?');
            return *this;
        }

        message& operator<< (wchar_t* chunk)
        {
            return operator<< (static_cast<const wchar_t*>(chunk));
        }

        message& operator<< (const std::string& chunk)
        {
            stream_ << chunk.c_str();
            return *this;
        }

        message& operator<< (const std::wstring& chunk)
        {
            *this << chunk.c_str();
            return *this;
        }

        message& operator << (const message& msg)
        {
            stream_ << msg.stream_.str().c_str();

            locale_ = msg.locale_;
            severity_ = msg.severity_;
            context_ = msg.context_;

            return *this;
        }

        std::string text() const
        {
            return stream_.str();
        }

        const char* severity_name() const
        {
            return level_name(severity_);
        }

        log_level severity() const
        {
            return severity_;
        }

        const context_info& context() const
        {
            return context_;
        }

    private:
        std::locale locale_;
        std::stringstream stream_;
        log_level severity_;
        context_info context_;
    };

    inline bool operator==(const message& msg1, const message& msg2)
    {
        if (msg1.severity_ == msg2.severity_ && msg1.stream_.str() == msg2.stream_.str())
            return true;
        else
            return false;
    }

    inline std::ostream& operator<< (std::ostream &stream, const message& msg)
    {
        stream << msg.severity_name() << ":" << msg.text();
        return stream;
    }
}



#endif