#ifndef PLOG_MONOAPPENDER_LOGGER_H_
#define PLOG_MONOAPPENDER_LOGGER_H_

#include <memory>

#include "plog/logger.h"
#include "plog/log_message.h"
#include "plog/appender.h"

#include "plog/csv_formatter.h"
#include "plog/formatter.h"

namespace plog
{
    class monoappender_logger 
        : public logger
    {
    public:
        monoappender_logger( appender* appender_instance /*takes ownership on appender_instance*/,/* )*/
                             formatter* formatter_instance =NULL /*takes ownership on formatter_instance*/ )
            : appender_(appender_instance), formatter_(formatter_instance)
        { 
            if(   ( appender_.get() != NULL )
                && ( appender_->newly_created() ) )
            {
                if (formatter_.get() == NULL)
                {
                    formatter_.reset(new plog::csv_formatter);
                }

                appender_->write( formatter_->header(), formatter_->header_size() );
            }
        }

        ~monoappender_logger(){};

        void write(const message& msg)
        {
            const size_t BUF_SIZE = 512;
            char buffer[BUF_SIZE];

            if( appender_.get() != NULL )
            {
                size_t length = formatter_->format(msg, buffer, BUF_SIZE);

                appender_->write(buffer, length);
            }
        }

        bool check_severity(log_level severity)
        {
            if( appender_.get() != NULL )
            {
                return (appender_->get_level() >= severity);
            }
            else
            {
                return false;
            }
        }

    private:
        std::auto_ptr<appender>     appender_;
        std::auto_ptr<formatter>    formatter_;
    };
}

#endif //PLOG_MONOAPPENDER_LOGGER_H_