#include "plog/csv_formatter.h"
#include "plog/str_utils.h"

#include <boost/format.hpp>
#include <algorithm>
#include <windows.h>
#include <cstdlib>

namespace plog
{
const char CSV_HEADER[] = "Date;Time;Severity;TID;File;This;Function;Message"NEWLINE;

size_t csv_formatter::format(const message& msg, char* buffer, size_t buffer_size) const
{
    const context_info& context = msg.context();

    const char* file_name = str_utils::base_name(context.file);


    int res = _snprintf_s(buffer, buffer_size, _TRUNCATE, "%04d/%02d/%02d;%02d:%02d:%02d:%03d;%s;%d;%s@%d;0x%p;%s;%s"NEWLINE,
        context.time.wYear, context.time.wMonth, context.time.wDay, 
        context.time.wHour, context.time.wMinute, context.time.wSecond, context.time.wMilliseconds,
        msg.severity_name(), context.tid, file_name, context.line, context.object, context.func_name, 
        msg.text().c_str());

    if (res == -1)
    {
        size_t len = buffer_size - 1;
        buffer[len - 2] = '\r';
        buffer[len - 1] = '\n';
        return len;
   }
    else
    {
        return res;
    }

}

size_t csv_formatter::header_size() const
{
    return sizeof(CSV_HEADER) - sizeof(char);
}

const char* csv_formatter::header() const
{
    return CSV_HEADER;
}

}