#include "plog/file_appender.h"
#include "plog/error_observer.h"

#include <sstream>

#include "file_utils.h"

namespace plog
{

//////////////////////////////////////////////////////////////////////////

file_appender::file_appender(log_level min_severity) 
: appender(min_severity)
, rewritten_(false)
{
    file_utils::init_file_utils();
}

file_appender::file_appender(const wchar_t* file_name, bool rewrite_file, log_level min_severity)
: appender(min_severity)
, file_name_(file_name)
{
    this->init(file_name_.c_str(), rewrite_file);
}

void file_appender::init(const wchar_t* file_name, bool rewrite_file)
{
    if (file_name_ != file_name)
    {
        file_name_ = file_name;
    }

    DWORD createDisposition = rewrite_file ? CREATE_ALWAYS : OPEN_ALWAYS;

    file_.Close();
    HRESULT hr = file_.Create(file_name,
                              FILE_APPEND_DATA | FILE_WRITE_ATTRIBUTES | SYNCHRONIZE,
                              FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
                              createDisposition,
                              FILE_ATTRIBUTE_NORMAL,
                              NULL);

    if( FAILED(hr))
    {
        get_error_observer()->on_error("Cannot open log file");
    }

    if (!rewrite_file && ::GetLastError() == ERROR_ALREADY_EXISTS)
    {
        rewritten_ = false;
    }
    else
    {
        rewritten_ = true;
    }
}

void file_appender::write(const void*  buffer, size_t length)
{
    if (file_.m_h != NULL)
    {
        if (this->is_file_deleted())
        {
            this->reinitialize_file();
        }
        HRESULT hr = file_.Write(buffer, static_cast<DWORD>(length));
        if (FAILED(hr))
        {
            get_error_observer()->on_error("Cannot write to file");
        }
    }
}

bool file_appender::newly_created()
{
    return rewritten_;
}

bool file_appender::is_file_deleted()
{
    return file_utils::is_deleted(file_.m_h);
}

void file_appender::reinitialize_file()
{
    if(file_.m_h == NULL)
    {
        get_error_observer()->on_error("file's handle is invalid");
    }
    else
    {
        this->init(file_name_.c_str(), true);
    }
}
} //namespace