
#include "plog/error_observer.h"

#include "file_utils.h"

// 
// If you have the ddk, include ntstatus.h.
// 
#ifndef STATUS_SUCCESS
#define STATUS_SUCCESS  ((NTSTATUS)0x00000000L)
#endif

#ifndef NT_SUCCESS
#define NT_SUCCESS(Status) ((NTSTATUS)(Status) >= 0)
#endif

namespace plog
{
    namespace file_utils
    {
        typedef enum _FILE_INFORMATION_CLASS 
        {
            FileStandardInformation = 5    
        } FILE_INFORMATION_CLASS, *PFILE_INFORMATION_CLASS;

        typedef struct _IO_STATUS_BLOCK 
        {
            union 
            {
                NTSTATUS Status;
                PVOID Pointer;
            } DUMMYUNIONNAME;

            ULONG_PTR Information;
        } IO_STATUS_BLOCK, *PIO_STATUS_BLOCK;

        typedef struct _FILE_STANDARD_INFORMATION 
        {
            LARGE_INTEGER AllocationSize;
            LARGE_INTEGER EndOfFile;
            ULONG         NumberOfLinks;
            BOOLEAN       DeletePending;
            BOOLEAN       Directory;
        } FILE_STANDARD_INFORMATION, *PFILE_STANDARD_INFORMATION;

        typedef NTSTATUS (NTAPI *NtQueryInformationFileFunctionType)(
            IN HANDLE FileHandle,
            OUT PIO_STATUS_BLOCK IoStatusBlock,
            OUT PVOID FileInformation,
            IN ULONG Length,
            IN FILE_INFORMATION_CLASS FileInformationClass);

        static NtQueryInformationFileFunctionType NtQueryInformationFile = NULL;

        bool is_deleted( HANDLE file_handle )
        {
            IO_STATUS_BLOCK ioStatus;
            NTSTATUS status;
            FILE_STANDARD_INFORMATION info;

            status = NtQueryInformationFile(file_handle, 
                &ioStatus, 
                &info, 
                sizeof(FILE_STANDARD_INFORMATION), 
                FileStandardInformation);

            if (status == STATUS_SUCCESS && 
                NT_SUCCESS(ioStatus.Status))
            {
                return info.DeletePending != 0;
            }

            return false;
        }

        void init_file_utils()
        {
            (FARPROC&)NtQueryInformationFile = GetProcAddress(GetModuleHandle(L"ntdll.dll"), 
                "NtQueryInformationFile");
            if (NtQueryInformationFile == NULL)
            {
                plog::get_error_observer()->on_error("Cannot get NtQueryInformationFile");
            }
        }
    }
}
