#define BOOST_TEST_MODULE plog_complex_test
#include <boost/test/unit_test.hpp>

#include <fstream>

#include <boost/thread.hpp>
#include <boost/regex.hpp>

#include "plog/log.h"

void test()
{
    for (int i = 0; i < 10; ++i)
    {
        LOG(plog::info) << "Hello, world!";
        LOG(plog::info) << "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.";
    }
}

BOOST_AUTO_TEST_SUITE(complex_test)

BOOST_AUTO_TEST_CASE(multithreaded_log)
{
    plog::init(L"test.log", plog::debug, true);

    //////////////////////////////////////////////////////////////////////////
    // create log
    boost::thread_group tg;

    for (int i = 0; i < 10; ++i)
    {
        tg.create_thread(test);
    }
    tg.join_all();

    //////////////////////////////////////////////////////////////////////////
    // verify log
    static const boost::regex expression("\\d+\\.\\d+\\.\\d+\t\\d+:\\d+:\\d+:\\d+\t\\[.{7}]\t[^\t]+\t[^\\[\\]0-9\t]+");

    std::fstream fs("test.log", std::fstream::in);
    while (!fs.eof())
    {
        char s[1024];
        fs.getline(s, 1024);

        if (fs.eof() && !strlen(s))
            return;

        boost::smatch match_result;
        bool matched = boost::regex_match(std::string(s), match_result, expression);
        BOOST_CHECK_MESSAGE(matched, s);
    }

}

BOOST_AUTO_TEST_SUITE_END()