#include <boost/test/unit_test.hpp>

#include "plog/log.h"

namespace
{
    struct mock_logger: public plog::custom_log
    {
        void log(plog::log_level level, plog::context_info context, const std::string& msg)
        {
            message_.reset(new plog::message(level, context, msg.c_str()));
        }

        std::auto_ptr<plog::message> message_;
    };

    struct fixture 
    {
        ~fixture()         
        { 
            plog::free(); 
        }
    };
}

BOOST_FIXTURE_TEST_SUITE(custom_logger_test, fixture)

BOOST_AUTO_TEST_CASE(test)
{
    mock_logger my_log;
    plog::init_custom_log(&my_log);

    plog::message msg(plog::info, CONTEXT_INFO(NULL));
    msg << "hello world: " << 1 << L"," << 2 << "," << 3;

    LOG(plog::info) << msg;

    BOOST_REQUIRE(my_log.message_.get() != NULL);
    BOOST_REQUIRE_EQUAL(msg, *my_log.message_);
}

BOOST_AUTO_TEST_SUITE_END()