#include <boost/test/unit_test.hpp>

#include "plog/log_message.h"

BOOST_AUTO_TEST_SUITE(message_test)

BOOST_AUTO_TEST_CASE(stream_output)
{
    std::string expectedSeverity("INFO");
    std::string expectedText("some text");

    plog::message msg(plog::info, plog::context_info(), "some text");

    BOOST_REQUIRE_EQUAL(expectedSeverity, msg.severity_name());
    BOOST_REQUIRE_EQUAL(expectedText, msg.text());
}

BOOST_AUTO_TEST_CASE(check_equal)
{
    plog::message expectedMessage(plog::info, plog::context_info(), "strawberry fields 4ever");

    plog::message msg(plog::info, plog::context_info());
    msg << "strawberry fields " << 4 << "ever";

    BOOST_REQUIRE_EQUAL(expectedMessage, msg);
}

BOOST_AUTO_TEST_CASE(check_different_severity)
{
    plog::message expectedMessage(plog::info, plog::context_info(), "boom");

    plog::message msg(plog::error, plog::context_info());
    msg << "boom";

    BOOST_REQUIRE_NE(expectedMessage, msg);
}

BOOST_AUTO_TEST_CASE(check_different_text)
{
    plog::message expectedMessage(plog::info, plog::context_info(), "foo");

    plog::message msg(plog::info, plog::context_info());
    msg << "bar";

    BOOST_REQUIRE_NE(expectedMessage, msg);
}

BOOST_AUTO_TEST_CASE(check_wide_char_const)
{
    plog::message expectedMessage(plog::info, plog::context_info(), "Hello World");

    plog::message msg(plog::info, plog::context_info());
    msg << L"Hello World";

    BOOST_REQUIRE_EQUAL(expectedMessage, msg);
}

BOOST_AUTO_TEST_CASE(check_wide_char_nonconst)
{
    plog::message expectedMessage(plog::info, plog::context_info(), "Hello World");

    wchar_t* nonconst = L"Hello World";
    plog::message msg(plog::info, plog::context_info());
    msg << nonconst;

    BOOST_REQUIRE_EQUAL(expectedMessage, msg);
}

BOOST_AUTO_TEST_CASE(check_wstring)
{
    plog::message expectedMessage(plog::info, plog::context_info(), "Hello World");

    plog::message msg(plog::info, plog::context_info());
    msg << std::wstring(L"Hello World");

    BOOST_REQUIRE_EQUAL(expectedMessage, msg);
}

BOOST_AUTO_TEST_SUITE_END()
