#define BOOST_TEST_NO_MAIN
#include <boost/test/unit_test.hpp>

#include "plog/logger_holder.h"

//////////////////////////////////////////////////////////////////////////
// logger MOC object

class MOC_logger : public plog::logger
{
public:
    MOC_logger() 
        : allow_destruction_(true)
    {
        BOOST_ERROR("MOC_logger default constructor must be never called");
        ++instances_count_;
    }

    /*you may call only this constructor*/
    MOC_logger(bool allow_destruction)
        : allow_destruction_(allow_destruction)
    {
        ++instances_count_;
    }

    void allow_destruction()
    {
        allow_destruction_ = true;
    }

    static void check_instances_count()
    {
        BOOST_REQUIRE_EQUAL(instances_count_, 0);
    }

    ~MOC_logger()
    {
        if( !allow_destruction_ )
        {
            BOOST_ERROR("MOC_logger destruction is not allowed at this point");
        }

        --instances_count_;
    }

    // plog::logger interface
    void write(const plog::message&)
    {
    }

    bool check_severity(plog::log_level)
    { 
        return false;
    }

private:
    bool allow_destruction_;
    static int  instances_count_;
};

int MOC_logger::instances_count_ = 0;

//////////////////////////////////////////////////////////////////////////
// tests

BOOST_AUTO_TEST_SUITE(non_singleton_log_test)

BOOST_AUTO_TEST_CASE(uninitialized_log)
{
    BOOST_REQUIRE_EQUAL( plog::logger_holder::get_logger(), (plog::logger*)NULL);
}

BOOST_AUTO_TEST_CASE(initialization_of_the_log)
{
    plog::logger* my_logger = new MOC_logger(true);
    plog::logger_holder::set_logger( my_logger );
    BOOST_REQUIRE_EQUAL( plog::logger_holder::get_logger(), my_logger );

    plog::logger_holder::set_logger(NULL);
    BOOST_REQUIRE_EQUAL( plog::logger_holder::get_logger(), (plog::logger*)NULL);

    MOC_logger::check_instances_count();
}

BOOST_AUTO_TEST_CASE(releasing_of_the_log)
{
    MOC_logger* my_logger = new MOC_logger(false);

    plog::logger_holder::set_logger( my_logger );
    BOOST_REQUIRE_EQUAL( plog::logger_holder::get_logger(), my_logger );
    
    my_logger = dynamic_cast<MOC_logger*>( plog::logger_holder::release_logger());
    BOOST_REQUIRE_EQUAL( plog::logger_holder::get_logger(), (plog::logger*)NULL);
    
    plog::logger_holder::set_logger(NULL);
    BOOST_REQUIRE_EQUAL( plog::logger_holder::get_logger(), (plog::logger*)NULL);

    my_logger->allow_destruction();

    delete my_logger;

    MOC_logger::check_instances_count();
}

BOOST_AUTO_TEST_SUITE_END()
