#include <boost/test/unit_test.hpp>

#include "plog/txt_formatter.h"


namespace
{
    const char short_msg[] = "short message is short";

    const char long_msg[] = 
        "long message is so \
looooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooong";
}

static const size_t BUF_SIZE = 512;
static char buffer[BUF_SIZE];

BOOST_AUTO_TEST_SUITE(txt_formatter)

BOOST_AUTO_TEST_CASE(format_short_message)
{
    SYSTEMTIME time = {1986, 9, 1, 1, 12, 0, 0, 0};
    plog::message msg(
        plog::info, 
        plog::context_info(__FILE__, __FUNCTION__, 777, 1234, NULL, &time), 
        short_msg);

    plog::txt_formatter formatter;
    formatter.format(msg, buffer, BUF_SIZE);

    std::stringstream expected;
    expected << "1986.09.01\t" << "12:00:00:000\t[   INFO]\t" << __FUNCTION__ << "\t" << short_msg << "\r\n";
    size_t expectedLength = expected.str().length();
    
    BOOST_CHECK_EQUAL(expectedLength, strlen(buffer));
    BOOST_CHECK_EQUAL(buffer[expectedLength - 2], '\r');
    BOOST_CHECK_EQUAL(buffer[expectedLength - 1], '\n');
    BOOST_CHECK_EQUAL(expected.str(), buffer);
}

BOOST_AUTO_TEST_CASE(format_long_message)
{
    SYSTEMTIME time = {1986, 9, 1, 1, 12, 0, 0, 0};
    plog::message msg(
        plog::info, 
        plog::context_info(__FILE__, __FUNCTION__, 0, 0, 0, &time), 
        long_msg);

    plog::txt_formatter formatter;
    size_t length = formatter.format(msg, buffer, BUF_SIZE);

    std::stringstream expected;
    expected << "1986.09.01\t" << "12:00:00:000\t[   INFO]\t" << __FUNCTION__ << "\t" << long_msg << "\r\n";
    size_t expectedLength = 511;

    BOOST_CHECK_EQUAL(expectedLength, strlen(buffer));
    BOOST_CHECK_EQUAL(expected.str().substr(0, expectedLength - 2), std::string(buffer).substr(0, expectedLength - 2));
    BOOST_CHECK_EQUAL(buffer[expectedLength - 2], '\r');
    BOOST_CHECK_EQUAL(buffer[expectedLength - 1], '\n');
    BOOST_CHECK_EQUAL(expectedLength, length);
}

BOOST_AUTO_TEST_SUITE_END()